package com.main.producer;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.main.entity.Employee;
import lombok.extern.slf4j.Slf4j;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.kafka.support.SendResult;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

@Component
@Slf4j
public class EmployeeProducer {

    private final KafkaTemplate<Integer, String> kafkaTemplate;
    public final ObjectMapper objectMapper;

    @Value("${spring.kafka.topic}")
    public String topic;

    public EmployeeProducer(KafkaTemplate<Integer, String> kafkaTemplate, ObjectMapper objectMapper) {
        this.kafkaTemplate = kafkaTemplate;
        this.objectMapper = objectMapper;
    }




    public CompletableFuture<SendResult<Integer, String>> sendEmployee(List<Employee> emp) throws JsonProcessingException {
        if (emp.isEmpty()) {
            throw new IllegalArgumentException("Employee list cannot be empty");
        }

        var key = emp.get(0).getId();
        var value = objectMapper.writeValueAsString(emp);

        return sendWithRetry(key, value, 3, 1000); // Retry up to 3 times with a backoff of 1000 ms
    }




    private CompletableFuture<SendResult<Integer, String>> sendWithRetry(Integer key, String value, int retries, long backoff) {
        return kafkaTemplate.send(topic, key, value).handle((sendResult, throwable) -> {
            if (throwable != null) {
                if (retries > 0 && throwable.getCause() instanceof org.apache.kafka.common.errors.NotLeaderOrFollowerException) {
                    log.warn("Retrying due to NOT_LEADER_OR_FOLLOWER error. Attempts left: {}", retries);
                    try {
                        TimeUnit.MILLISECONDS.sleep(backoff);
                    } catch (InterruptedException e) {
                        Thread.currentThread().interrupt();
                        log.error("Retry interrupted", e);
                    }
                    return sendWithRetry(key, value, retries - 1, backoff).join(); // Retry
                } else {
                    handleFailure(key, value, throwable);
                    return null; // Give up on failure
                }
            } else {
                handleSuccess(key, value);
                return sendResult;
            }
        });
    }


    public CompletableFuture<SendResult<Integer, String>> sendEmployee(Employee emp) throws JsonProcessingException {


        var key = emp.getId();
        var value = objectMapper.writeValueAsString(emp);

        return kafkaTemplate.send(topic, key , value);// Retry up to 3 times with a backoff of 1000 ms
    }










    private void handleSuccess(Integer key, String value ) {
        log.info("Message sent successfully for the key: {}, value: {} ", key, value);
    }

    private void handleFailure(Integer key, String value, Throwable ex) {
        log.error("Error sending the message: {}, exception: {}", value, ex.getMessage(), ex);
    }
}
